use std::collections::HashMap;
use crate::prelude::{RetroCallbacks, RetroComponent, InputDeviceId, InputIndex};

#[derive(Default)]
pub struct StatefulInputComponent {
    states: Vec<HashMap<(InputDeviceId, InputIndex), i16>>,
}

impl StatefulInputComponent {
    pub fn set_input(&mut self, port: u32, device: InputDeviceId, index: impl Into<Option<InputIndex>>, value: i16) {
        while port >= self.states.len() as u32 {
            self.states.push(Default::default());
        }
        let index = index.into().unwrap_or(InputIndex::Left);
        self.states[port as usize].insert((device, index), value);
    }
}

impl RetroComponent for StatefulInputComponent {}

impl RetroCallbacks for StatefulInputComponent {
    fn input_state(&mut self, port: u32, device: InputDeviceId, index: InputIndex) -> i16 {
        self.states.get(port as usize)
            .and_then(|x| x.get(&(device, index)))
            .map(|x| *x)
            .unwrap_or_default()
    }
}
