use crate::prelude::*;

/// Write's the core's own log statements to stderr.
pub struct StderrLogComponent {
    /// May be set to add a prefix to each logged line.
    pub prefix: String,
    /// The most verbose log level that should be printed.
    pub level: ferretro_base::retro::ffi::LogLevel,
}

impl Default for StderrLogComponent {
    fn default() -> Self {
        StderrLogComponent {
            prefix: String::new(),
            level: ferretro_base::retro::ffi::LogLevel::Info,
        }
    }
}

impl RetroComponent for StderrLogComponent {}

impl RetroCallbacks for StderrLogComponent {
    fn log_print(&mut self, level: ferretro_base::retro::ffi::LogLevel, msg: &str) {
        if level >= self.level {
            eprint!("{}[{:?}] {}", self.prefix, level, msg);
        }
    }
}
