use std::path::Path;
use crate::prelude::*;

/// Writes all the input descriptors, variables, and subsystem information to stderr as they are
/// provided by the core.
#[derive(Default)]
pub struct StderrSysInfoLogComponent {
    /// May be set to add a prefix to each logged line.
    pub prefix: String,
}

impl RetroComponent for StderrSysInfoLogComponent {
    fn post_load_game(&mut self, retro: &mut LibretroWrapper, _rom: &Path) -> crate::base::Result<()> {
        self.set_system_av_info(&retro.get_system_av_info());
        Ok(())
    }
}

impl RetroCallbacks for StderrSysInfoLogComponent {
    fn set_memory_maps(&mut self, memory_map: &MemoryMap) -> Option<bool> {
        eprintln!("{}{:?}", self.prefix, memory_map);
        Some(true)
    }

    fn set_geometry(&mut self, game_geometry: &GameGeometry) -> Option<bool> {
        eprintln!("{}{:?}", self.prefix, game_geometry);
        Some(true)
    }

    fn set_system_av_info(&mut self, system_av_info: &SystemAvInfo) -> Option<bool> {
        eprintln!("{}{:?}", self.prefix, system_av_info);
        Some(true)
    }

    fn set_input_descriptors(&mut self, descriptors: &[InputDescriptor2]) -> Option<bool> {
        for id in descriptors {
            eprintln!("{}{:?}", self.prefix, id);
        }
        Some(true)
    }

    fn set_variables(&mut self, variables: &[Variable2]) -> Option<bool> {
        for v in variables {
            eprintln!("{}{:?}", self.prefix, v);
        }
        Some(true)
    }

    fn set_subsystem_info(&mut self, subsystem_info: &[SubsystemInfo2]) -> Option<bool> {
        for s in subsystem_info {
            eprintln!("{}{:?}", self.prefix, s);
        }
        Some(true)
    }
}
